package com.compdfkit.pdfviewer.customview.listview;

import android.content.Context;
import android.graphics.Canvas;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Scroller;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageButton;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.recyclerview.widget.DividerItemDecoration;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.watermark.CPDFWatermark;
import com.compdfkit.pdfviewer.activities.WatermarkActivity;
import com.compdfkit.pdfviewer.R;
import com.compdfkit.pdfviewer.adapter.WatermarkListAdapter;
import com.compdfkit.pdfviewer.entity.WatermarkSetting;
import com.compdfkit.pdfviewer.fragments.WatermarkConsoleFragment;
import com.compdfkit.pdfviewer.tools.WatermarkHandle;

public class WatermarkListView extends ConstraintLayout {
    private boolean isExpand = true;
    private int listWidth;

    private Context context;

    private final RecyclerView watermarkList;
    private WatermarkListAdapter adapter;
    private WatermarkConsoleFragment consoleFragment;

    private final AppCompatImageButton expandBtn;

    private Scroller mScroller;

    @Override
    public void computeScroll() {
        super.computeScroll();
        if (mScroller.computeScrollOffset()) {
            scrollTo(mScroller.getCurrX(), mScroller.getCurrY());
            invalidate();
        }
    }

    public void smoothScrollX(int destX) {
        int scrollX = getScrollX();
        int deltaX = destX - scrollX;
        mScroller.startScroll(scrollX, 0, deltaX, 0, 500);
        invalidate();
    }

    public WatermarkListView(@NonNull Context context) {
        this(context, null);
    }

    public WatermarkListView(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public WatermarkListView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        this(context, attrs, defStyleAttr, 0);
    }

    public WatermarkListView(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);

        this.context = context;
        this.mScroller = new Scroller(context);

        mScroller = new Scroller(context);
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);

        View currentView = inflater.inflate(R.layout.watermark_list_view, this, true);

        watermarkList = currentView.findViewById(R.id.watermark_list);
        watermarkList.post(() -> listWidth = watermarkList.getWidth());

        expandBtn = currentView.findViewById(R.id.expand_list_btn);

        expandBtn.setOnClickListener(v -> {
            if (isExpand) {
                hintList();
            } else {
                expandWatermarkList();
            }
        });
    }

    public void expandWatermarkList() {
        showView();

        WatermarkActivity activity = (WatermarkActivity) consoleFragment.getActivity();
        assert activity != null;

        try {
            activity.expandWatermarkList();
        } catch (CPDFDocumentException e) {
            e.printStackTrace();
        }
    }

    public void initList(CPDFDocument document, WatermarkConsoleFragment consoleFragment) {
        int watermarkCount = document.getWatermarkCount();

        if (!WatermarkHandle.getEnable()) {
            for (int i = 0; i < watermarkCount; i++) {
                CPDFWatermark tempWatermark = document.getWatermark(i);
                WatermarkHandle.addSettingIntoList(tempWatermark);
            }
        }

        this.consoleFragment = consoleFragment;
        adapter = new WatermarkListAdapter(context, consoleFragment, WatermarkHandle.getSettingList());
        watermarkList.setAdapter(adapter);

        LinearLayoutManager manager = new LinearLayoutManager(context, LinearLayoutManager.VERTICAL, false);
        watermarkList.setLayoutManager(manager);
        watermarkList.addItemDecoration(new DividerItemDecoration(context, DividerItemDecoration.VERTICAL));

        setItemTouchHelper();
    }

    private void setItemTouchHelper() {
        ItemTouchHelper.Callback callback = new ItemTouchHelper.Callback() {
            int offset = 0;
            float maxDX = 0;
            boolean isExpand = false;
            boolean isFirstSwipe = false;

            @Override
            public void onChildDraw(@NonNull Canvas c, @NonNull RecyclerView recyclerView, @NonNull RecyclerView.ViewHolder viewHolder, float dX, float dY, int actionState, boolean isCurrentlyActive) {
                WatermarkListAdapter.WatermarkItemHolder holder = (WatermarkListAdapter.WatermarkItemHolder) viewHolder;

                if (dX == 0.0f) {
                    offset = viewHolder.itemView.getScrollX();
                    isFirstSwipe = true;
                }

                if (isCurrentlyActive) {
                    viewHolder.itemView.scrollTo(offset - (int) dX, 0);
                    isExpand = viewHolder.itemView.getScrollX() > holder.getDeleteViewOffset();
                } else {
                    if (isFirstSwipe) {
                        maxDX = dX;
                        isFirstSwipe = false;
                    }

                    if (isExpand)
                        viewHolder.itemView.scrollTo(Math.max(offset - (int) dX - 5, holder.getDeleteViewOffset()), 0);
                    else {
                        if (maxDX > offset) {
                            viewHolder.itemView.scrollTo(-(int) dX, 0);
                            return;
                        }

                        if (offset >= holder.getDeleteViewOffset())
                            viewHolder.itemView.scrollTo((int) ((offset - (int) maxDX) * (dX / maxDX)), 0);
                        else
                            viewHolder.itemView.scrollTo(offset - (int) dX - 2, 0);
                    }
                }
            }

            @Override
            public float getSwipeThreshold(@NonNull RecyclerView.ViewHolder viewHolder) {
                return Integer.MAX_VALUE;
            }

            @Override
            public float getSwipeEscapeVelocity(float defaultValue) {
                return Integer.MAX_VALUE;
            }

            @Override
            public int getMovementFlags(@NonNull RecyclerView recyclerView, @NonNull RecyclerView.ViewHolder viewHolder) {
                int dragFlags = 0;

                int swipeFlags = ItemTouchHelper.LEFT | ItemTouchHelper.RIGHT;
                return makeMovementFlags(dragFlags, swipeFlags);
            }

            @Override
            public boolean onMove(@NonNull RecyclerView recyclerView, @NonNull RecyclerView.ViewHolder viewHolder, @NonNull RecyclerView.ViewHolder target) {
                return true;
            }

            @Override
            public void onSwiped(@NonNull RecyclerView.ViewHolder viewHolder, int direction) {
                WatermarkActivity activity = (WatermarkActivity) consoleFragment.getActivity();
                assert activity != null;

                try {
                    activity.deleteWatermark(viewHolder.getAdapterPosition());
                } catch (CPDFDocumentException e) {
                    e.printStackTrace();
                }

                adapter.deleteSettingFromList(viewHolder.getAdapterPosition());
            }
        };
        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(callback);
        itemTouchHelper.attachToRecyclerView(watermarkList);
    }

    public void addSettingIntoList(WatermarkSetting setting) {
        adapter.addSettingIntoList(setting);
    }

    public void showView() {
        smoothScrollX(0);
        expandBtn.setRotation(180);
        isExpand = true;
    }

    public void hintView() {
        smoothScrollX(getWidth());
    }

    public void hintList() {
        smoothScrollX(listWidth);
        expandBtn.setRotation(0);
        isExpand = false;
    }
}

